package java112.analyzer;

import java.io.*;
import java.util.*;

/**
 * Analyzes text files by tokenizing and passing to specific analyzers.
 * Manages all analyzers. Reads input file one line at a time and
 * splits each line into individual text tokens on regex non-word
 * characters. Calls each analyzers processing method for each
 * collected token. Calls output report method for all analyzers.
 * @author Brian Manning
 */
public class AnalyzeFile {



    //Constant, stores expected number of command line arguments.
    private static final int CORRECT_ARGUMENT_NUMBER = 2;


    //Stores all analyzers to be run
    private List<Analyzer> analyzers;

    //Stores a string representing path of the file being analyzed.
    private String inputFilePath;

    //Properties file for all analyzers.
    private Properties properties;

    /**
     * Manages all processing. Tests for expected commandline arguments,
     * halts on failure. Opens input file and splits into individual text
     * tokens. Iterates through all tokens and calls processing methods
     * for each analyzer. When processing is complete calls output report
     * methods for each analyzer.
     * @param arguments String[] - command line arguments passed
     * by AnalyzerDriver containing path of input file.
     */
    public void runAnalysis(String[] arguments) {
        if (arguments.length != CORRECT_ARGUMENT_NUMBER) {
            System.out.println(
                    "Enter path to input file, path to properties file");
            return;
        }
        inputFilePath = arguments[0];
        loadProperties(arguments[1]);

        startAnalyzers();
        openInputFile();
        writeAllOutputFiles();
    }

    /**
     * Loads the properties file for use with all analyzers
     * @param propertiesFilePath String - path to properties file
     */
    public void loadProperties(String propertiesFilePath)  {
        properties = new Properties();
        try {
            properties.load(this.getClass().getResourceAsStream(propertiesFilePath));
        }
        catch(IOException ioe) {
            System.out.println("Can't load the properties file");
            ioe.printStackTrace();
        }
        catch(Exception e) {
            System.out.println("Problem: " + e);
            e.printStackTrace();
        }
    }

    /**
     * Opens the input file for processing.
     */
    private void openInputFile() {
        BufferedReader inputFile = null;

        try {
            inputFile = new BufferedReader(new FileReader(inputFilePath));
            readInputLine(inputFile);
        } catch (java.io.FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (java.io.IOException exception) {
            exception.printStackTrace();
        } catch (Exception exception) {
            exception.printStackTrace();
        } finally {
            try {
                if (inputFile != null) {
                    inputFile.close();
                }
            } catch (java.io.IOException exception) {
                exception.printStackTrace();
            } catch (Exception exception) {
                exception.printStackTrace();
            }
        }
    }

    /**
     * Iterates through all strings in list of text tokens, test for empty
     * strings, passes each token to processing methods for analyzers.
     * @param tokenList String[] - List of all individual tokens
     * in input line.
     */
    private void processTokenList(String[] tokenList) {
        for (String token : tokenList) {
            if (!token.isEmpty()) {
                processTokenForAllAnalyzers(token);
            }
        }
    }

    /**
     * Iterates through all analyzers and calls processing for the token
     * @param token String - An individual text token.
     */
    private void processTokenForAllAnalyzers(String token) {
        for (Analyzer analyzer : analyzers) {
            analyzer.processToken(token);
        }
    }

    /**
     * Reads input file one line at a time and passes lines to
     * splitLineOnWord.
     * @param inputFile BufferedReader - Contents of input file.
     */
    private void readInputLine(BufferedReader inputFile)
            throws IOException, Exception {

        String inputLine;

        while (inputFile.ready()) {
            inputLine = inputFile.readLine();
            splitLineOnWord(inputLine);
        }
    }

    /**
     * Splits a single line from input file on regex non-word characters.
     * Passes to processTokenList for processing by analyzers.
     * @param line String - A single line of the input file.
     */
    private void splitLineOnWord(String line) {
        String[] lineList = line.split("\\W");
        processTokenList(lineList);
    }

    /**
     * Instantiates all analyzers and adds to the list.
     */
    private void startAnalyzers() {
        analyzers = new ArrayList<Analyzer>();
        analyzers.add(new UniqueTokenAnalyzer(properties));
        analyzers.add(new SummaryReport(properties));
        analyzers.add(new BigWordAnalyzer(properties));
        analyzers.add(new TokenCountAnalyzer(properties));
        analyzers.add(new KeywordAnalyzer(properties));
        analyzers.add(new TokenSizeAnalyzer(properties));
    }

    /**
     * Calls output report methods for all analyzers.
     */
    private void writeAllOutputFiles() {
        for (Analyzer analyzer : analyzers) {
            analyzer.writeOutputFile(inputFilePath);
        }
    }
}
