package java112.analyzer;

import java.util.*;
import java.io.*;

/**
 * Processes individual text tokens. Compares tokens to list of keywords
 * Records the position in the token sequence of occurances of keywords.
 * Writes a report of positions for each keyword.
 * @author Brian Manning
 */
public class KeywordAnalyzer implements Analyzer {

    // Properties file for this analyzer.
    private Properties properties;

    // stores the position of the token in the sequence of all tokens
    private int tokenOccurence;

    // Stores a map of text token and integer occurance number pairs.
    private Map<String, List<Integer>> keywordMap;

    /**
     * Instantiates a TreeMap object as tokenCounts on creation.
     */
    public KeywordAnalyzer() {
        keywordMap = new TreeMap<String, List<Integer>>();
    }

    /**
     * Constructor for KeywordAnalyzer analyzer.
     * @param properties Properties - Properties file values for this analyzer.
     */
    public KeywordAnalyzer(Properties properties) {
        this();
        this.properties = properties;
        openKeywordFile();
    }

    /**
     * Accessor for the keywordMap instance variable.
     * @return keywordMap Map. A map of all keyword tokens paired with a list
     * of positions.
     */
    public Map<String, List<Integer>> getKeywordMap() {
        return keywordMap;
    }

    /**
     * Processes a single token. Tests for presence in map, adds current
     * position to list if present.
     * @param token String - a single text token.
     */
    public void processToken(String token) {
        tokenOccurence++;
        if (keywordMap.containsKey(token)) {
            List<Integer> positions = keywordMap.get(token);
            positions.add(tokenOccurence);
        }
    }

    /**
     * Writes a text file listing each keyword and a list of the positions in
     * which it occurs.
     * @param inputFilePath String - Path of the file being analyzed.
     */
    public void writeOutputFile(String inputFilePath) {
        PrintWriter tokenReport = null;

        try {
            tokenReport = new PrintWriter( new BufferedWriter(new FileWriter(getOutputFilePath())));
            writeKeywordList(tokenReport);

        } catch (java.io.FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (java.io.IOException exception) {
            exception.printStackTrace();
        } catch (Exception exception) {
            exception.printStackTrace();
        } finally {
            if (tokenReport != null) {
                tokenReport.close();
            }
        }
    }

    /**
     * Retrieves values from properties for path to output file.
     * @return outputFilePath String - Path of the output file.
     */
    public String getOutputFilePath() {
        return properties.getProperty("output.dir") + properties.getProperty("output.file.keyword");
    }

    /**
     * Loops through all entries in the keywordMap map and writes each keyword
     * followed by a list of all the positions in which it occurs.
     * @param keywordReport PrintWriter - Output file for the report.
     */
    private void writeKeywordList(PrintWriter keywordReport) {
        for (Map.Entry<String, List<Integer>> entry : keywordMap.entrySet()) {

            String keyword = entry.getKey();
            keywordReport.println(keyword + " =");

            List<String> lines = processKeywordPostions(entry.getValue());
            writeKeywordListLines(keywordReport, lines);

            keywordReport.println("");
        }
    }

    /**
     * Loops through all entries in the lines List and writes each line to
     * the report.
     * @param keywordReport PrintWriter - Output file for the report.
     * @return List<String> lines- List of formatted token position report lines.
     */
    private void writeKeywordListLines(PrintWriter keywordReport, List<String> lines) {
        for (String line : lines) {
            keywordReport.println(line);
        }
    }

    /**
     * Tests for and empty positions list and returns correctly formated empty
     * list report. If not empty passes the positions list to 
     * processPositionListLines for formatting.
     * @param List<String> - List of correctly formatted lines for report.
     * @return List<String> lines- List of formatted token position report lines.
     */
    private List<String> processKeywordPostions(List<Integer> positions) {
        List<String> lines = null;

        if (positions.isEmpty()) {
            lines = new ArrayList<String>();
            lines.add("[]");
            return lines;
        }

        lines = processPositionListLines(positions);
        return lines;
    }

    /**
     * Formats the keyword positions for report output. Begins with an opening
     * bracket on the first line. Adds positions followed by a comma and space
     * to the line until MAX_LINE_LENGTH is exceeded. When MAX_LINE_LENGTH is
     * exceeded trims the trailing space, adds to lines List, and starts a new
     * line. After all positions have been processed trims the trailing space
     * the final line and add it to the line list. Finally passes to 
     * correctPositionListLastLine for correction of the final line of the
     * osition list.
     * @param List<Integer> lines- List of token positions.
     * @return List<String> lines- List of formatted token position report lines.
     */
    private List<String> processPositionListLines(List<Integer> positions) {
        final int MAX_LINE_LENGTH = 75;
        List<String> lines = new ArrayList<String>();

        String line = "[";

        for (Integer position : positions) {

            line += position + ", ";

            if (line.length() > MAX_LINE_LENGTH) {
                lines.add(line.trim());
                line = "";
            }
        }

        lines.add(line.trim());
        lines = correctPositionListLastLine(lines);
        return lines;
    }

    /**
     * Replaces the trailing comma with a closing bracket on the last line
     * of the position list. Handles extra empty last lines by removing from list
     * @param List<String> lines- List of formatted token position report lines.
     * @return List<String> lines- List of formatted token position report lines.
     */
    private List<String> correctPositionListLastLine(List<String> lines) {
        String lastLine = lines.get(lines.size() - 1);

        if (lastLine.isEmpty()) {
            lines.remove(lines.size() - 1);
            lastLine = lines.get(lines.size() - 1);
        }

        lastLine = lastLine.substring(0, lastLine.length() - 1) + "]";

        lines.set(lines.size() - 1, lastLine);
        return lines;
    }

    /**
     * Opens the keyword file for processing.
     */
    private void openKeywordFile() {
        BufferedReader inputFile = null;

        try {
            inputFile = new BufferedReader(new FileReader( properties.getProperty("file.path.keywords")));
            readInputLine(inputFile);
        } catch (java.io.FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (java.io.IOException exception) {
            exception.printStackTrace();
        } catch (Exception exception) {
            exception.printStackTrace();
        } finally {
            try {
                if (inputFile != null) {
                    inputFile.close();
                }
            } catch (java.io.IOException exception) {
                exception.printStackTrace();
            } catch (Exception exception) {
                exception.printStackTrace();
            }
        }
    }

    /**
     * Reads keyword file one line at a time and adds entries to keywordMap
     * @param inputFile BufferedReader - Contents of input file.
     */
    private void readInputLine(BufferedReader inputFile)
            throws IOException, Exception {

        String line = null;
        while (inputFile.ready()) {
            line = inputFile.readLine();
            if (!line.isEmpty()) {
                keywordMap.put(line, new ArrayList<Integer>());
            }
        }
        /** QUESTION: Following throws null pointer exception when run. Why?
         while (inputFile.ready()) {
            if (!inputFile.readLine().isEmpty()) {
                keywordMap.put(inputFile.readLine(), new ArrayList<Integer>());
            }
         }
         */
    }

}
