package java112.analyzer;

import java.util.*;
import java.io.*;

/**
 * Processes individual text tokens and tallies total tokens in file.
 * Writes a summary report including timestamp and total text tokens.
 * @author Brian Manning
 */
public class SummaryReport implements Analyzer {

    //Properties file for this analyzer.
    private Properties properties;

    //Stores tally of individual text tokens.
    private int totalTokensCount;

    /**
     * empty constructor for SummaryReport analyzer.
     */
    public SummaryReport() {

    }

    /**
     * Constructor for SummaryReport analyzer.
     * @param properties Properties - Properties file values for this analyzer.
     */
    public SummaryReport(Properties properties) {
        this.properties = properties;
    }

    /**
     * Accessor for the totalTokensCount instance variable.
     * @return totalTokensCount, the total number of processed tokens.
     */
    public int getTotalTokensCount() {
        return totalTokensCount;
    }

    /**
     * Processes a single token by incrementing totalTokensCount.
     * @param token String - a single text token.
     */
    public void processToken(String token) {
        totalTokensCount++;
    }

    /**
     * Manages report creation. Writes a simpe summary of the file analysis.
     * @param inputFilePath String - Path of the file being analyzed.
     */
    public void writeOutputFile(String inputFilePath) {

        String outputFilePath = getOutputFilePath();
        PrintWriter summaryReport = null;

        try {
            summaryReport = new PrintWriter(
                new BufferedWriter(new FileWriter(outputFilePath)));
            writeReportContent(summaryReport, inputFilePath);
        } catch (java.io.FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (java.io.IOException exception) {
            exception.printStackTrace();
        } catch (Exception exception) {
            exception.printStackTrace();
        } finally {
            if (summaryReport != null) {
                summaryReport.close();
            }
        }
    }

    /**
     * Retrieves values from properties for path to output file.
     * @return outputFilePath String - Path of the output file.
     */
    public String getOutputFilePath() {
        return properties.getProperty("output.dir")
                + properties.getProperty("output.file.summary");
    }

    /**
     * Writes a text file summary report including timestamp and total tokens.
     * @param summaryReport PrintWriter - File for report output.
     * @param inputFilePath String - Path of the file being analyzed.
     */
    public void writeReportContent(PrintWriter summaryReport,
            String inputFilePath)
            throws IOException, Exception {

        Date timeStamp = new Date();

        summaryReport.println("Application: "
                + properties.getProperty("application.name"));
        summaryReport.println("Author: "
                + properties.getProperty("author") + ", "
                + properties.getProperty("author.section"));
        summaryReport.println("email: "
                + properties.getProperty("author.email.address"));
        summaryReport.println("Input file: " + inputFilePath);
        summaryReport.println("Analyzed on: " + timeStamp);
        summaryReport.println("Total token count: " + totalTokensCount);
    }

}
