package java112.analyzer;

import java.util.*;
import java.io.*;

/**
 * Processes individual text tokens and tallies the count of each unique token.
 * Writes a report including each token and the number of occurances.
 * @author Brian Manning
 */
public class TokenCountAnalyzer implements Analyzer {


    //Properties file for this analyzer.
    private Properties properties;


    //Stores a map of text token and integer occurance number pairs.
    private Map<String, Integer> tokenCounts;

    /**
     * Instantiates a TreeMap object as tokenCounts on creation.
     */
    public TokenCountAnalyzer() {
        tokenCounts = new TreeMap<String, Integer>();
    }

    /**
     * Constructor for TokenCountAnalyzer analyzer. Instantiates the
     * tokenCounts Map as TreeMap.
     * @param properties Properties - Properties file values for this analyzer.
     */
    public TokenCountAnalyzer(Properties properties) {
        this();
        this.properties = properties;
    }

    /**
     * Accessor for the tokenCounts instance variable.
     * @return tokenCounts Map. A map of all unique tokens and occurance
     * integer pairs.
     */
    public Map getTokenCounts() {
        return tokenCounts;
    }

    /**
     * Processes a single token. Tests for presence in map, adds if
     * not present, increments if present.
     * @param token String - a single text token.
     */
    public void processToken(String token) {

        if (tokenCounts.containsKey(token)) {
            int value = tokenCounts.get(token);
            tokenCounts.put(token, ++value);
        } else {
            tokenCounts.put(token, 1);
        }
    }

    /**
     * Writes a text file listing each token, quantity pair on a new line.
     * @param inputFilePath String - Path of the file being analyzed.
     */
    public void writeOutputFile(String inputFilePath) {
        PrintWriter tokenReport = null;

        try {
            tokenReport = new PrintWriter(
                    new BufferedWriter(new FileWriter(getOutputFilePath())));
            writeTokenList(tokenReport);

        } catch (java.io.FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (java.io.IOException exception) {
            exception.printStackTrace();
        } catch (Exception exception) {
            exception.printStackTrace();
        } finally {
            if (tokenReport != null) {
                tokenReport.close();
            }
        }
    }

    /**
     * Retrieves values from properties for path to output file.
     * @return outputFilePath String - Path of the output file.
     */
    public String getOutputFilePath() {
        return properties.getProperty("output.dir")
                + properties.getProperty("output.file.token.count");
    }

    /**
     * Loops through all entries in the tokenCounts map and writes
     * a line with tab separated token, count pairs.
     * @param uniqueTokenReport PrintWriter - Output file for the
     * unique token report.
     */
    private void writeTokenList(PrintWriter tokenReport) {
        for (Map.Entry entry : tokenCounts.entrySet()) {
            tokenReport.println(entry.getKey() + "\t" + entry.getValue());
        }
    }

}
