package java112.analyzer;

import java.util.*;
import java.io.*;
/**
 * Processes individual text tokens and tallies the number of tokens for each
 * distinct token length. Writes a report listing each token length followed by
 * the number of occurances. Draws a simple text histogram of length frequency
 * in horizontal and vertical orientations.
 * @author Brian Manning
 */
public class TokenSizeAnalyzer implements Analyzer {
    // map that stores token length and the number of occurances for that length
    private Map<Integer, Integer> tokenSizes;
    // stores properties for this analyzer.
    private Properties properties;
    //stores the size of the largest token.
    private int maximumSize;

    /**
     * Constructor for TokenSizeAnalyzer. Instantiates the
     * tokenSizes Map as TreeMap.
     */
    public TokenSizeAnalyzer() {
        tokenSizes = new TreeMap<Integer, Integer>();
    }

    /**
     * Constructor for TokenSizeAnalyzer.
     * @param properties Properties - Properties file values for this analyzer.
     */
    public TokenSizeAnalyzer(Properties properties) {
        this();
        this.properties = properties;
    }

    /**
     * Accessor for the tokenSizes instance variable.
     * @return tokenSizes Map. Map of all token sizes and number of occurances.
     */
    public Map<Integer, Integer> getTokenSizes() {
        return tokenSizes;
    }

    /**
     * Accessor for the maximumSize instance variable.
     * @return maximumSize int. The size of the largest token.
     */
    public int getMaximumSize() {
        return maximumSize;
    }

    /**
     * Processes a single token. Tests for presence of token size in map,
     * adds if not present, increments if present.
     * @param token String - a single text token.
     */
    public void processToken(String token) {
        if (tokenSizes.containsKey(token.length())) {
            int value = tokenSizes.get(token.length());
            tokenSizes.put(token.length(), ++value);
        } else {
            tokenSizes.put(token.length(), 1);
        }
    }

    /**
     * Writes a text file listing each length, quantity pair on a new line.
     * @param inputFilePath String - Path of the file being analyzed.
     */
    public void writeOutputFile(String inputFilePath) {
        PrintWriter tokenReport = null;

        try {
            tokenReport = new PrintWriter(
                    new BufferedWriter(new FileWriter(getOutputFilePath())));
            writeTokenSizeReport(tokenReport);
            tokenReport.println("");
            writeTokenHistogram(tokenReport);
            tokenReport.println("");
            writeVerticalTokenHistogram(tokenReport);
        } catch (java.io.FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (java.io.IOException exception) {
            exception.printStackTrace();
        } catch (Exception exception) {
            exception.printStackTrace();
        } finally {
            if (tokenReport != null) {
                tokenReport.close();
            }
        }
    }
    /**
     * Writes a text file listing each length, quantity pair on a new line.
     * @param tokenReport PrintWriter - Path of the file being analyzed.
     */
    private void writeTokenSizeReport(PrintWriter tokenReport) {
        for (Map.Entry<Integer, Integer> entry : tokenSizes.entrySet()) {
            tokenReport.println(entry.getKey() + "\t" + entry.getValue());
        }
    }

    /**
     * Writes to a text file a histogram composed of asterisks reflecting the
     * values for each key in the tokenSizes map.
     * @param tokenReport PrintWriter - Path of the file being analyzed.
     */
    private void writeTokenHistogram(PrintWriter tokenReport) {
        //value for number of columns in report
        final int MAX_LINE_LENGTH = 75;
        //value for number of spaces before graph starts
        final int WHITESPACE_BUFFER = 4;
        double mostTokens = Collections.max(tokenSizes.values());
        double scale = mostTokens / MAX_LINE_LENGTH;
        for (Map.Entry<Integer, Integer> entry : tokenSizes.entrySet()) {
            String line = addCharacters(
                    entry.getKey().toString(),
                    WHITESPACE_BUFFER - entry.getKey().toString().length(),
                    ' ');
            line = addCharacters(
                    line,
                    (int) Math.ceil(entry.getValue() / scale),
                    '*');
            tokenReport.println(line);
        }
    }

    /**
     * Adds the specified character the specified number of times to
     * the input string.
     * @param input String - string to add characters to.
     * @param quantity quantity - number of characters to add.
     * @param character char - character to add.
     * @return input String. The input string with specified charcters added.
     */
    private static String addCharacters (String input, int quantity, char character) {
          char[] repeat = new char[quantity];
          Arrays.fill(repeat, character);
          input += new String(repeat);
          return input;
    }

    /**
     * Writes to a text file. Creates a histogram composed of asterisks
     * reflecting the values for each key in the tokenSizes map in vertical
     * orientation.
     * @param tokenReport PrintWriter - Path of the file being analyzed.
     */
    private void writeVerticalTokenHistogram(PrintWriter tokenReport) {
        //value for number of rows in report
        final int MAX_COLUMN_HEIGHT = 34;
        //value for number of spaces for each row entry
        final int WHITESPACE_BUFFER = 3;
        double mostTokens = Collections.max(tokenSizes.values());
        double scale = mostTokens / MAX_COLUMN_HEIGHT;
        int currentRow = (int) Math.ceil(mostTokens / scale);
        while (currentRow > 0) {
            tokenReport.println(buildVerticalHistogramRow(
                    currentRow,
                    scale,
                    WHITESPACE_BUFFER));
            currentRow--;
        }
        tokenReport.println(buildVerticalHistogramFooter(WHITESPACE_BUFFER));
    }
    /**
     * Loops throught the tokenSizes map and creates a row for the vertical
     * histogram by assigning asterisk or space to the position for that token
     * length Draws for the histogram row passed in by currentRow.
     * @param currentRow int - the current row of the vertical histogram.
     *      Start from the top.
     * @param scale double - the scale factor used to adjust the raw number of
     *      token occurances.
     * @param entryLength int - the exact character size for each entry in the
     *      row.
     * @return line String - The completed row of the histogram.
     */
    private String buildVerticalHistogramRow(int currentRow, double scale,
            int entryLength) {
        String line = "";
        String marker = "*";
        for (int value : tokenSizes.values()) {
            if ((int) Math.ceil(value / scale) >= currentRow) {
                line += addCharacters(
                        marker, entryLength - marker.length(), ' ');
            } else {
                line = addCharacters(line, entryLength, ' ');
            }
        }
        return line;
    }

    /**
     * Loops through all the keys in the tokenSizes map and formats the token
     * length values as labels for the vertical histogram.
     * @param entryLength int - exact character size for each entry in the row.
     * @return line String - The completed footer of the histogram.
     */
    private String buildVerticalHistogramFooter(int entryLength) {
        String line = "";
        for (Integer key : tokenSizes.keySet()) {
            line += addCharacters(
                    key.toString(),
                    entryLength - key.toString().length(),
                    ' ');
        }
        return line;
    }

    /**
     * Retrieves values from properties for path to output file.
     * @return outputFilePath String - Path of the output file.
     */
    public String getOutputFilePath() {
        return properties.getProperty("output.dir")
                + properties.getProperty("output.file.token.size");
    }
}
