package java112.analyzer;

import java.util.*;
import java.io.*;

/**
 * Processes individual text tokens and stores a list of unique values.
 * Writes a report listing one unique token per line.
 * @author Brian Manning
 */
public class UniqueTokenAnalyzer implements Analyzer {


    //Properties file for this analyzer.
    private Properties properties;

    //Stores a list of unique text tokens.
    private Set uniqueTokensList;

    /**
     * Instantiates a TreeSet object as uniqueTokensList on creation.
     */
    public UniqueTokenAnalyzer() {
        uniqueTokensList = new TreeSet();
    }

    /**
     * Constructor for UniqueTokenAnalyzer analyzer. Instantiates the
     * uniqueTokensList Set as TreeSet.
     * @param properties Properties - Properties file values for this analyzer.
     */
    public UniqueTokenAnalyzer(Properties properties) {
        this();
        this.properties = properties;
    }

    /**
     * Accessor for the uniqueTokensList instance variable.
     * @return uniqueTokenList Set. A list of all unique tokens.
     */
    public Set getUniqueTokensList() {
        return uniqueTokensList;
    }

    /**
     * Processes a single token by adding the token to uniqueTokensList.
     * @param token String - a single text token.
     */
    public void processToken(String token) {
        uniqueTokensList.add(token);
    }

    /**
     * Writes a text file listing each unique token on a new line.
     * @param inputFilePath String - Path of the file being analyzed.
     */
    public void writeOutputFile(String inputFilePath) {

        String outputFilePath = getOutputFilePath();
        PrintWriter uniqueTokenReport = null;

        try {
            uniqueTokenReport = new PrintWriter(
                    new BufferedWriter(new FileWriter(outputFilePath)));
            writeTokenList(uniqueTokenReport);

        } catch (java.io.FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (java.io.IOException exception) {
            exception.printStackTrace();
        } catch (Exception exception) {
            exception.printStackTrace();
        } finally {
            if (uniqueTokenReport != null) {
                uniqueTokenReport.close();
            }
        }
    }

    /**
     * Retrieves values from properties for path to output file.
     * @return outputFilePath String - Path of the output file.
     */
    public String getOutputFilePath() {
        return properties.getProperty("output.dir")
                + properties.getProperty("output.file.unique");
    }

    /**
     * Loops through all items in the uniqueTokensList and writes
     * a line for each to the output file.
     * @param uniqueTokenReport PrintWriter - Output file for the
     * unique token report.
     */
    private void writeTokenList(PrintWriter uniqueTokenReport) {
        for (Iterator iterator =
                uniqueTokensList.iterator(); iterator.hasNext();) {
            uniqueTokenReport.println(iterator.next());
        }
    }

}
