package java112.employee;

import java.util.*;
import java.sql.*;
import java.io.*;

/**
 * class EmployeeDirectory manages queries and inserts on the employee DB.
 * @author Brian Manning
 */
public class EmployeeDirectory {

    private Properties properties;

    /**
     * Empty Constructor for EmployeeDirectory
     */
    public EmployeeDirectory() {

    }

    /**
     * Constructor for EmployeeDirectory with properties.
     * Loads DB driver
     */
    public EmployeeDirectory(Properties properties) {
        this.properties = properties;
        loadDbDriver();
    }

    /**
     * Executes query based on search string and field from the search object
     *@param  Search search - the search object with values for this search
     *@return Search search - The search object with all results added
     */
    public Search executeSearch(String searchType, String searchTerm) {
        Search search = new Search();

        search.setSearchTerm(searchTerm);
        search.setSearchType(searchType);

        if (searchType.equals("employeeId")) {
            return searchEmployeeId(search);
        } else if (searchType.equals("lastName")) {
            return searchEmployeeLastName(search);
        }
        return search;
    }

    /**
     * Assembles the select statement for a search on employeeId. passes to
     * searchDB for processing.
     * @param Search search - search object containing search string.
     * @return search - The search object with all results added.
     */
    public Search searchEmployeeId(Search search) {

        String sql = "SELECT emp_id, first_name, last_name, ssn, dept, "
                + "room, phone FROM employees WHERE emp_id LIKE '"
                + search.getSearchTerm() + "%'";
        search = searchDb(search, sql);
        return search;
    }

    /**
     * Assembles the select statement for a search on lastName. passes to
     * searchDB for processing.
     * @param Search search - search object containing search string.
     * @return search - The search object with all results added.
     */
    public Search searchEmployeeLastName(Search search) {

        String sql = "SELECT emp_id, first_name, last_name, ssn, dept, "
                + "room, phone FROM employees WHERE last_name LIKE '"
                + search.getSearchTerm() + "%'";
        search = searchDb(search, sql);
        return search;
    }

    /**
     * Queries employee table based on provided sql string.
     * Manages db connection.
     * @param Search search - search object containing search string.
     * @param String sql - sql select statement
     * @return search - The search object with all results added.
     */
    private Search searchDb(Search search, String sql) {
        Connection connection = null;
        Statement statement = null;
        ResultSet resultSet = null;

        try {
            connection = openDbConnection();
            statement = connection.createStatement();
            resultSet = statement.executeQuery(sql);

            search = parseEmployeeRows(search, resultSet);

        } catch (ClassNotFoundException classNotFound) {
            System.err.println("Cannot find database driver ");
            classNotFound.printStackTrace();
        } catch (SQLException sqlException) {
            System.err.println("Error connecting to database " + sqlException);
            sqlException.printStackTrace();
        } catch (Exception exception) {
            System.err.println("General Error");
            exception.printStackTrace();
        } finally {
            try {

                if (statement != null) {
                    statement.close();
                }

                if (connection != null) {
                    connection.close();
                }
            } catch (SQLException sqlException) {
                System.err.println("Error connecting to database "
                        + sqlException);
                sqlException.printStackTrace();
            } catch (Exception exception) {
                System.err.println("General Error");
                exception.printStackTrace();
            }
        }
        return search;
    }

    /**
     * Reads result set and adds records returned by query to search object
     * @param Search search - search object containing search string.
     * @param ResultSet resultSet - Result set for a query.
     * @return search - The search object with all results added.
     */
    private Search parseEmployeeRows(Search search, ResultSet resultSet) 
            throws SQLException, Exception {

        while (resultSet.next()) {
            search.setEmployeesFound(true);
            Employee employee = new Employee();
            employee.setEmployeeId(resultSet.getString("emp_id"));
            employee.setFirstName(resultSet.getString("first_name"));
            employee.setLastName(resultSet.getString("last_name"));
            employee.setSocialSecurityNumber(resultSet.getString("ssn"));
            employee.setDepartment(resultSet.getString("dept"));
            employee.setRoomNumber(resultSet.getString("room"));
            employee.setPhoneNumber(resultSet.getString("phone"));
            search.addFoundEmployee(employee);
        }

        return search;
    }

    /**
     * Assembles the insert statement to add a record to the employee table.
     * Generates a sucess/fail message based on rows altered.
     * searchDB for processing.
     * @param String firstName - value for the firstName field.
     * @param String lastName - value for the lastName field.
     * @param String socialSecurityNumber - value for the socialSecurity field.
     * @param String department - value for the department field.
     * @param String roomNumber - value for the roomNumber field.
     * @param String phone - value for the phone field.
     * @return String - Success/fail message.
     */
    public String addEmployee(String firstName, String lastName,
            String socialSecurityNumber, String department, String roomNumber,
            String phoneNumber) {

        int rowsAffected;
        String sql = "INSERT INTO employees"
                + "(first_name, last_name, ssn, dept, room, phone)"
                + " VALUES('" + firstName + "', '"
                + lastName + "', '" + socialSecurityNumber + "', '"
                + department + "', '" + roomNumber + "', '"
                + phoneNumber + "')";

        rowsAffected = updateDb(sql);
        if (rowsAffected > 0) {
            return "Employee Added";
        }
        return "Failed: No Employee Added";
    }

    /**
     * Inserts into employee table based on provided sql string.
     * Manages db connection.
     * @param String sql - sql insert statement.
     * @return int - The number of rows altered.
     */
    private int updateDb(String sql) {
        Connection connection = null;
        Statement statement = null;
        int rowsAffected = 0;

        try {
            connection = openDbConnection();
            statement = connection.createStatement();
            rowsAffected = statement.executeUpdate(sql);

        } catch (ClassNotFoundException classNotFound) {
            System.err.println("Cannot find database driver ");
            classNotFound.printStackTrace();
        } catch (SQLException sqlException) {
            System.err.println("Error connecting to database " + sqlException);
            sqlException.printStackTrace();
        } catch (Exception exception) {
            System.err.println("General Error");
            exception.printStackTrace();
        } finally {
            try {
                if (statement != null) {
                    statement.close();
                }

                if (connection != null) {
                    connection.close();
                }
            } catch (SQLException sqlException) {
                System.err.println("Error connecting to database "
                        + sqlException);
                sqlException.printStackTrace();
            } catch (Exception exception) {
                System.err.println("General Error");
                exception.printStackTrace();
            }
        }
        return rowsAffected;
    }

    /**
     * Opens the connection to the DB. Retrieves parameters from properties.
     * @return Connection - the DB connection object.
     */
    private Connection openDbConnection() throws SQLException, Exception {
        Connection connection = DriverManager.getConnection(
                properties.getProperty("db.url"),
                properties.getProperty("db.user"),
                properties.getProperty("db.password"));
        return connection;
    }

    /**
     * Loads DB driver specified in properties.
     */
    private void loadDbDriver() {
        try {
            Class.forName(properties.getProperty("db.driver"));
        } catch (ClassNotFoundException classNotFound) {
            System.err.println("Cannot find database driver ");
            classNotFound.printStackTrace();
        } catch (Exception exception) {
            System.err.println("General Error");
            exception.printStackTrace();
        }
    }

}
